package com.care.bms.service;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.bigscreen.service.BigScreenService;
import com.care.common.entity.CareDevice;
import com.care.common.entity.CareEventOrder;
import com.care.common.entity.CareHouse;
import com.care.common.entity.CareMqttMsg;
import com.care.common.enums.OrderStatusEnum;
import com.care.common.enums.OrderTypeEnum;
import com.care.common.service.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author: stw
 * @Date: 2021/8/14
 * @Desc:
 */
@Service
public class MqttMsgService {
    @Resource
    private CareHouseService careHouseService;
    @Resource
    private CareStationService careStationService;
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareEventOrderService careEventOrderService;
    @Resource
    private CareOrganizationService careOrganizationService;
    @Resource
    private BigScreenService bigScreenService;
    @Resource
    private CareMqttMsgService careMqttMsgService;


    public void handleMessage(Map<String,Thread> threadMap,String topic, String mqttMessageStr) throws Exception {
        try {
            QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(CareDevice::getTopic,topic);
            CareDevice careDevice = careDeviceService.getOne(queryWrapper);
            if(careDevice != null) {
                CareMqttMsg careMqttMsg = new CareMqttMsg();
                careMqttMsg.setMqttMessage(mqttMessageStr);
                careMqttMsg.setTopic(topic);
                careMqttMsg.setDevId(careDevice.getId());
                careMqttMsg.setCreateTime(new Date());
                this.careMqttMsgService.save(careMqttMsg);
            } else {
                System.out.println("数据库中没找到对应设备");
                return;
            }

            Map map = JSON.parseObject(mqttMessageStr);
            String type = (String) map.get("type");
            Map msgMap = (Map)map.get("msg");
            if ("BreathHeartRate".equals(type)) {
                Map msg = (Map) map.get("msg");
                Integer breath = (Integer) msg.get("breath");
                Integer heart = (Integer) msg.get("heart");
                careDevice.setBreathRate(breath);
                careDevice.setHeartRate(heart);
                careDeviceService.updateById(careDevice);
            } else if ("FallDown".equals(type)) { //跌到
                Integer conf = (Integer)msgMap.get("conf");
                if(conf == 255){ //取消
                    threadMap.forEach((key,value) -> {
                        if(key.startsWith("FallDown")) {
                            if(!value.isInterrupted()){
                                value.interrupt();//停止线程
                                threadMap.remove(key);
                                value = null;
                            }
                        }
                    });
                } else {
                    Integer fallResponseTime = careDevice.getFallResponseTime();
                    MyThread myThread = new MyThread();
                    myThread.setCareDevice(careDevice);
                    myThread.setOrderType(OrderTypeEnum.DI_EDAO.getValue());
                    myThread.setSleepTime(fallResponseTime);
                    myThread.setDate(DateUtil.date());
                    threadMap.put("FallDown-" + System.currentTimeMillis(),myThread);
                    myThread.start();
                }


            } else if ("StayTooLong".equals(type)) { //久滞
                Integer conf = (Integer)msgMap.get("conf");
                if(conf == 255){ //取消
                    threadMap.forEach((key,value) -> {
                        if(key.startsWith("StayTooLong")) {
                            if(!value.isInterrupted()){
                                value.interrupt();//停止线程
                                threadMap.remove(key);
                                value = null;
                            }
                        }
                    });
                } else {
                    Integer longlagResponseTime = careDevice.getLonglagResponseTime();
                    MyThread myThread = new MyThread();
                    myThread.setCareDevice(careDevice);
                    myThread.setOrderType(OrderTypeEnum.JIU_ZHI.getValue());
                    myThread.setSleepTime(longlagResponseTime);
                    myThread.setDate(DateUtil.date());
                    threadMap.put("StayTooLong-" + System.currentTimeMillis(),myThread);
                    myThread.start();
                }

            }
        } catch (Exception e){
            e.printStackTrace();
        }
        System.out.println(topic);
    }

    private void saveOrder(CareDevice careDevice,String orderType,Date date){
        CareHouse careHouse = careHouseService.getById(careDevice.getHouseId());

        CareEventOrder order = new CareEventOrder();
        order.setOrgId(careHouse.getOrgId());
        order.setOrgName(careOrganizationService.getById(careHouse.getOrgId()).getName());
        order.setStationId(careHouse.getStationId());
        order.setStationName(careStationService.getById(careHouse.getStationId()).getName());
        order.setHouseId(careHouse.getId());
        order.setDevId(careDevice.getId());
        order.setDevName(careDevice.getDevName());
        order.setHouseName(careHouse.getName());
        order.setMemberId(careDevice.getMemberId());
        order.setTitle(careHouse.getAddr());
        order.setOrderType(orderType);
        order.setStatus(OrderStatusEnum.TODO.getValue());
        order.setCreateTime(date);
        order.setModifyTime(date);

        //判断房屋是否有未完成的告警事件
        QueryWrapper<CareEventOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareEventOrder::getHouseId,order.getHouseId()).and(
                wrapper-> wrapper.eq(CareEventOrder::getStatus, OrderStatusEnum.TODO.getValue())
                .or().eq(CareEventOrder::getStatus,OrderStatusEnum.DOING.getValue()));

        CareEventOrder orderDb = careEventOrderService.getOne(queryWrapper);
        careEventOrderService.saveOrder(order,orderDb);
        if(orderDb != null){//有，不再生成新的告警工单，只作为子事件插入到当前工单的历史记录中。
            //通知页面
            this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"orderUpdate");
        } else {  //新增，生成新的告警事件工单
            //通知页面
            this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"orderAdd");
        }

        this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"bigscreenRefresh");
    }

    class MyThread extends Thread {
        private CareDevice careDevice;
        private String orderType;
        private Integer sleepTime; //分钟
        private Date date;

        public Date getDate() {
            return date;
        }

        public void setDate(Date date) {
            this.date = date;
        }

        public void setCareDevice(CareDevice careDevice) {
            this.careDevice = careDevice;
        }

        public void setOrderType(String orderType) {
            this.orderType = orderType;
        }

        public void setSleepTime(Integer sleepTime) {
            this.sleepTime = sleepTime;
        }


        @Override
        public void run(){
            try {
                if(sleepTime != null && sleepTime > 0){
                    Thread.sleep(sleepTime * 60 * 1000);
                }
                saveOrder(careDevice,orderType,date);
            } catch (Exception e){
                e.printStackTrace();
            }

        }

    }

}
